@@ -1,4 +1,5 @@
-import sqlite3, json
+import sqlite3, json, re
+# This is Trakos..
 def jsonify(data):
     
     json_string = data
@@ -8,24 +9,25 @@ class Client:
     def __init__(self,username):
         self.username = username
         self.connection = sqlite3.connect('database.db', check_same_thread=False)
-        self.cursor = self.connection.cursor()
+        
         
     def __getitem__(self, key):
-        return TableAPI(self.username, key, self.connection, self.cursor)
+        return TableAPI(self.username, key, self.connection)
         
         
 class TableAPI:
-    def __init__(self,username,table_name, connection, cursor):
+    def __init__(self,username,table_name, connection):
         self.username = username
         self.table_name = table_name
         self.connection = connection
-        self.cursor = cursor
+        
     def create_table(self):
         try:
-            self.cursor.execute(f"SELECT count(name) FROM sqlite_master WHERE type='table' AND name LIKE '{self.username}_%'")
-            count = self.cursor.fetchone()[0]
+            c = self.connection.cursor()
+            c.execute(f"SELECT count(name) FROM sqlite_master WHERE type='table' AND name LIKE '{self.username}_%'")
+            count = c.fetchone()[0]
             if count < 5:
-                self.cursor.execute(f"CREATE TABLE IF NOT EXISTS {self.username}_{self.table_name} (key TEXT PRIMARY KEY, value TEXT)")
+                c.execute(f"CREATE TABLE IF NOT EXISTS {self.username}_{self.table_name} (key TEXT PRIMARY KEY, value TEXT)")
                 self.connection.commit()
                 return jsonify(True)
             else:
@@ -35,14 +37,16 @@ class TableAPI:
   
             
     def set(self, key, value):
-        self.cursor.execute(f"INSERT OR REPLACE INTO {self.username}_{self.table_name} (key, value) VALUES (?, ?)", (key, json.dumps(value)))
+        c = self.connection.cursor()
+        c.execute(f"INSERT OR REPLACE INTO {self.username}_{self.table_name} (key, value) VALUES (?, ?)", (key, json.dumps(value)))
         self.connection.commit()
         return jsonify(True)
 
     def get(self,key):
         try:
-            self.cursor.execute(f"SELECT value FROM {self.username}_{self.table_name} WHERE key=?", (key,))
-            result = self.cursor.fetchone()
+            c = self.connection.cursor()
+            c.execute(f"SELECT value FROM {self.username}_{self.table_name} WHERE key=?", (key,))
+            result = c.fetchone()
             if result:
                 return jsonify(json.loads(result[0]))
             else:
@@ -52,7 +56,8 @@ class TableAPI:
 
     def delete(self, key):
         try:
-            self.cursor.execute(f"DELETE FROM {self.username}_{self.table_name} WHERE key = ?", (key,))
+            c = self.connection.cursor()
+            c.execute(f"DELETE FROM {self.username}_{self.table_name} WHERE key = ?", (key,))
             self.connection.commit()
             return jsonify(True)
         except sqlite3.OperationalError:
@@ -60,8 +65,9 @@ class TableAPI:
             
     def get_all(self):
         try:
-            self.cursor.execute(f"SELECT * FROM {self.username}_{self.table_name}")
-            data = self.cursor.fetchall()
+            c = self.connection.cursor()
+            c.execute(f"SELECT * FROM {self.username}_{self.table_name}")
+            data = c.fetchall()
             if data:
                 result = []
                 for item in data:
@@ -82,31 +88,45 @@ class TableAPI:
             return jsonify(str(e))
     def tables(self):
         try:
-            self.cursor.execute(f"SELECT name FROM sqlite_master WHERE type='table' AND name LIKE '{self.username}_%'")
-            tables = self.cursor.fetchall()
+            c = self.connection.cursor()
+            c.execute(f"SELECT name FROM sqlite_master WHERE type='table' AND name LIKE '{self.username}_%'")
+            tables = c.fetchall()
             self.connection.close()
             prefix = len(f"{self.username}_")
             user_tables = [table[0][prefix:] for table in tables]
             return user_tables
         except sqlite3.OperationalError:
             return None    
-    def keys(self,pattern=None):
+    
+
+    def keys(self,pattern=None,list=None):
         try:
-            self.cursor.execute(f"SELECT key FROM {self.username}_{self.table_name}")
-            data = self.cursor.fetchall()
+            c = self.connection.cursor()
+            c.execute(f"SELECT key FROM {self.username}_{self.table_name}")
+            data = c.fetchall()
             if data:
                 keys = []
                 for item in data:
-                    keys.append(item[0])
-                return jsonify("\n".join(keys))
+                    if pattern:
+                        pattern = pattern.replace("*", ".*")
+                        if re.match(pattern, item[0]):
+                            keys.append(item[0])
+                    else:
+                        keys.append(item[0])
+                if list:
+                    return keys
+                else:
+                    return jsonify("\n".join(keys))
+                
             else:
                 return jsonify(None)
         except Exception as e:
             return jsonify(str(e))
     
     def key_exists(self, key):
-            self.cursor.execute(f"SELECT 1 FROM {self.username}_{self.table_name} WHERE key=?", (key,))
-            result = self.cursor.fetchone()
+            c = self.connection.cursor()
+            c.execute(f"SELECT 1 FROM {self.username}_{self.table_name} WHERE key=?", (key,))
+            result = c.fetchone()
             if result:
                 return 1
             else:
@@ -114,13 +134,14 @@ class TableAPI:
     
     def push(self, key, value):
         try:
-            self.cursor.execute(f"SELECT value FROM {self.username}_{self.table_name} WHERE key=?", (key,))
-            data = self.cursor.fetchone()
+            c = self.connection.cursor()
+            c.execute(f"SELECT value FROM {self.username}_{self.table_name} WHERE key=?", (key,))
+            data = c.fetchone()
             if data:
                 json_data = json.loads(data[0])
                 if isinstance(json_data, list):
                     json_data.append(value)
-                    self.cursor.execute(f"UPDATE {self.username}_{self.table_name} SET value = ? WHERE key = ?", (json.dumps(json_data), key))
+                    c.execute(f"UPDATE {self.username}_{self.table_name} SET value = ? WHERE key = ?", (json.dumps(json_data), key))
                     self.connection.commit()
                     return True
                 else:
@@ -132,13 +153,14 @@ class TableAPI:
     
     def unpush(self, key, value):
         try:
-            self.cursor.execute(f"SELECT value FROM {self.username}_{self.table_name} WHERE key=?", (key,))
-            data = self.cursor.fetchone()
+            c = self.connection.cursor()
+            c.execute(f"SELECT value FROM {self.username}_{self.table_name} WHERE key=?", (key,))
+            data = c.fetchone()
             if data:
                 json_data = json.loads(data[0])
                 if isinstance(json_data, list):
                     json_data.remove(value)
-                    self.cursor.execute(f"UPDATE {self.username}_{self.table_name} SET value = ? WHERE key = ?", (json.dumps(json_data), key))
+                    c.execute(f"UPDATE {self.username}_{self.table_name} SET value = ? WHERE key = ?", (json.dumps(json_data), key))
                     self.connection.commit()
                     return True
                 else:
@@ -147,3 +169,4 @@ class TableAPI:
                 return "Key does not exist"
         except ValueError:
             return "Value is not in list"
+    
